CODIGO  SEGMENT
        ASSUME  CS:CODIGO
ENTRADA PROC    FAR
COMMENT *
===================================================================
=                                                                 =
=                 PROCEDIMIENTO BORRA_VENTANA                     =
=                          Versin 4b                             =
===================================================================

Descripcin: Borra una ventana definida dentro de la pantalla.

Condicin de llamada:
BH: Coordenada de la fila de la esquina superior izquierda
BL: Coordenada de la columna de la esquina superior izquierda
AH: Coordenada de la fila de la esquina inferior izquierda
AL: Coordenada de la columna de la esquina inferior izquierda
DH: Atributo que se asignar a la ventana borrada

Condicin de salida: 
C=0 si se ha borrado la ventana
C=1 si no se ha borrado la ventana por incoherencia de las  coordenadas pasadas
    como parmetros

Recursos usados y afectados:
AX: Para contener el par carcter/atributo
BX: Para ampliar a palabra el octeto CESI, en BL
CX: Para gestionar la iteracin del bucle interno
DL: Para gestionar la iteracin del bucle externo
DH: Para salvar temporalmente el parmetro BH
SI: Para contener el factor de iteracin del bucle interno
DI: Para direccionar la memoria de vdeo
BP: Para indicar el factor de actualizacin de DI, para que apunte al inicio
    de la siguiente lnea de la ventana
Registro de estado *

; -------------------------------------------------------------------

;Definicin de smbolos:
SEGMENTO_DE_VIDEO	EQU	0B800h 
ESPACIO		EQU	20h
ATRIBUTO	EQU	07h
FILAS		EQU	25
COLUMNAS	EQU	80

;Definicin alternativa de los registros de la CPU:
FEID		EQU	AH	; Fila de la esquina inferior derecha
CEID		EQU	AL	; Columna de la esquina inferior derecha
FESI		EQU	BH	; Fila de la esquina inferior izquierda
FESI_BIS	EQU	DH
CESI		EQU	BL	; Columna de la esquina inferior izquierda
CESI_BIS	EQU	BX	; Como CESI, pero en tamao palabra
ATRIBU		EQU	DX
CARACTER	EQU	AL
CARACTER_ATRIBUTO EQU AX
FACTOR_INCREMENTO_DI EQU BP
FACTOR_ITERACION_BUCLE_INT EQU	SI
CONTADOR_BUC_INT	EQU	CX
CONT_BUC_INT	EQU	CL
CONTADOR_BUC_EXT	EQU	DL
PUNTERO_DE_VIDEO	EQU 	DI


BORRA_VENTANA:
; Comprobar la coherencia de los parmetros pasados:
        CMP     FEID,FESI               ; FEID>=FESI?
	JB	INCOHERENTES
        CMP     CEID,CESI               ; CEID>=CESI?
	JB	INCOHERENTES
        CMP     FEID,FILAS              ; FEID>=25?
	JAE	INCOHERENTES
        CMP     CEID,COLUMNAS           ; CEID>=80?
	JAE	INCOHERENTES
        CMP     FESI,FILAS              ; FESI>=25?
	JAE	INCOHERENTES
        CMP     CESI,COLUMNAS           ; CESI>=80?
	JNAE	COHERENTES
INCOHERENTES:
; Retornar indicando error de parmetros:
	STC
	RET
COHERENTES:
; Guarda el atributo pasado por DH:
	PUSH	ATRIBU
; Inicia segmento de vdeo, en espacio de datos:
	MOV	DI,SEGMENTO_DE_VIDEO
	MOV	ES,DI
; Factor de iteracin del bucle externo <-- FEID-FESI+1:
	MOV	CONTADOR_BUC_EXT,FEID
	SUB	CONTADOR_BUC_EXT,FESI
	INC	CONTADOR_BUC_EXT
; Factor de iteracin del bucle interno <-- CEID-CESI+1
	MOV	FESI_BIS,FESI     ; salva FESI, que se necesita luego
	SUB	FESI,FESI	  ; BH puede re-usarse (BL a palabra BX)
	SUB	CONTADOR_BUC_INT,CONTADOR_BUC_INT
	MOV	CONT_BUC_INT,CEID
	SUB	CONT_BUC_INT,CESI
	INC	CONTADOR_BUC_INT
	MOV	FACTOR_ITERACION_BUCLE_INT,CONTADOR_BUC_INT
; Apuntar al inicio de la ventana, con DI, <--(160xFESI)+(CESIx2)
	MOV	AL,COLUMNAS*2
        MUL     FESI_BIS        ; en DH se salv FESI, que estaba en BH
	SHL	CESI_BIS,1
	ADD	AX,CESI_BIS
	MOV	PUNTERO_DE_VIDEO,AX
; Modo incremental del ndice DI:
	CLD
; BP <-- factor de actualizacin de DI, para apuntar a siguiente
; lnea: BP <-- 160-((CEID-CESI)x2):  
	MOV	FACTOR_INCREMENTO_DI,COLUMNAS*2	
	SHL	CONTADOR_BUC_INT,1
	SUB	FACTOR_INCREMENTO_DI,CX
; AX <-- carcter/atributo:
	POP	CARACTER_ATRIBUTO  ; recuperar en AH el atributo
	MOV	CARACTER,ESPACIO
BORRA_OTRA_FILA:
; recupera factor de iteracin del bucle interno: 
	MOV	CONTADOR_BUC_INT,FACTOR_ITERACION_BUCLE_INT
BORRA_FILA:	
	REP STOSW
; Apuntar al  inicio de la fila, con DI <-- DI+(160-(CEID-CESI)x2)
	ADD	PUNTERO_DE_VIDEO,FACTOR_INCREMENTO_DI
; quedan filas por borrar?
	DEC	CONTADOR_BUC_EXT
	JNZ	BORRA_OTRA_FILA
; NO: retornar:
	RET

ENTRADA ENDP
CODIGO  ENDS
        END ENTRADA
