CODIGO  SEGMENT
        ASSUME  CS:CODIGO
ENTRADA PROC    FAR
COMMENT *
===================================================================
=                                                                 =
=                 PROCEDIMIENTO BORRA_VENTANA                     =
=                          Versin 4c                             =
===================================================================

Descripcin: Borra una ventana definida dentro de la pantalla.

Condicin de llamada:
BH: Coordenada de la fila de la esquina superior izquierda
BL: Coordenada de la columna de la esquina superior izquierda
AH: Coordenada de la fila de la esquina inferior izquierda
AL: Coordenada de la columna de la esquina inferior izquierda
DH: Atributo que se asignar a la ventana borrada

Condicin de salida: 
C=0 si se ha borrado la ventana
C=1 si no se ha borrado la ventana por incoherencia de las  coordenadas
    pasadas como parmetros

Recursos usados y afectados:
AX: Para contener el par carcter/atributo
BX: Para ampliar a palabra el octeto CESI, en BL
CX: Para gestionar la iteracin del bucle interno
DL: Para gestionar la iteracin del bucle externo
DH: Para salvar temporalmente el parmetro BH
SI: Para contener el factor de iteracin del bucle interno
DI: Para direccionar la memoria de vdeo
BP: Para indicar el factor de actualizacin de DI, para que apunte al inicio
    de la siguiente lnea de la ventana
Registro de estado *

; -------------------------------------------------------------------

;Definicin de smbolos:
SEGMENTO_DE_VIDEO EQU 0B800h
ESPACIO         EQU     20h
FILAS		EQU	25
COLUMNAS	EQU	80

;Definicin alternativa de los registros de la CPU:
FEID		EQU	AH	; Fila de la esquina inferior derecha
CEID		EQU	AL	; Columna de la esquina inferior derecha
FESI		EQU	BH	; Fila de la esquina inferior izquierda
FESI_BIS	EQU	DH
CESI		EQU	BL	; Columna de la esquina inferior izquierda
CESI_BIS        EQU     BX      ; Como CESI, pero en tamao palabra
ATRIBUTO_	EQU	DX
CARACTER	EQU	AL
CARACTER_ATRIBUTO EQU AX
FACTOR_INCREMENTO_DI EQU BP
FACTOR_ITERACION_BUCLE_INT EQU	SI
CONTADOR_BUC_INTERNO	EQU	CX
CONT_BUC_INTERNO	EQU	CL
CONTADOR_BUC_EXTERNO	EQU	DL
PUNTERO_DE_VIDEO	EQU 	DI

;Definicin alternativa de las instrucciones:
TRANSFIERE_A	EQU	MOV
COPIA_EN	EQU	MOV
COMPARA 	EQU	CMP
GUARDA_EN_PILA	EQU	PUSH
SACA_DE_PILA	EQU	POP
SUMA_A		EQU	ADD
PON_A_CERO	EQU	SUB 
RESTA_A		EQU	SUB
INCREMENTA	EQU	INC
DECREMENTA	EQU	DEC
MULTIPLICA	EQU	MUL
;MULTIPLICA_A   EQU     SHL     ; no porque SHL se tomara como un operador
BORRA_ELEMENTO	EQU	STOSW
RETORNA 	EQU	RET
SI_NO_ES_MAYOR_NI_IGUAL_VE_A	EQU	JB
SI_ES_MAYOR_O_IGUAL_VE_A	EQU	JAE
SI_NO_ES_CERO_VE_A              EQU     JNZ
LOS_PARAMETROS_SON_INCOHERENTES EQU  STC
MODO_INCREMENTAL_DE_LOS_INDICES EQU  CLD
POR_DOS		EQU	1
VARIABLE_FESI	EQU	FESI,FESI
VARIABLE_CONT_BUC_INTERNO   EQU     CONTADOR_BUC_INTERNO,CONTADOR_BUC_INTERNO


BORRA_VENTANA:
; Comprobar la coherencia de los parmetros pasados:
	COMPARA	FEID,FESI
	SI_NO_ES_MAYOR_NI_IGUAL_VE_A INCOHERENTES
	COMPARA	CEID,CESI
	SI_NO_ES_MAYOR_NI_IGUAL_VE_A INCOHERENTES
	COMPARA	FEID,FILAS
	SI_ES_MAYOR_O_IGUAL_VE_A INCOHERENTES
	COMPARA	CEID,COLUMNAS
	SI_ES_MAYOR_O_IGUAL_VE_A INCOHERENTES
	COMPARA	FESI,FILAS	
	SI_ES_MAYOR_O_IGUAL_VE_A INCOHERENTES
	COMPARA	CESI,COLUMNAS
	SI_NO_ES_MAYOR_NI_IGUAL_VE_A COHERENTES
INCOHERENTES:
; Retornar indicando error de parmetros:
        LOS_PARAMETROS_SON_INCOHERENTES
        RETORNA
COHERENTES:
; Guarda el atributo pasado por DH:
	GUARDA_EN_PILA	ATRIBUTO_
; Inicia segmento de vdeo, en espacio de datos:
	TRANSFIERE_A	DI,SEGMENTO_DE_VIDEO
	TRANSFIERE_A	ES,DI
; Factor de iteracin del bucle externo <-- FEID-FESI+1:
	TRANSFIERE_A	CONTADOR_BUC_EXTERNO,FEID
	RESTA_A 	CONTADOR_BUC_EXTERNO,FESI
	INCREMENTA	CONTADOR_BUC_EXTERNO
; Factor de iteracin del bucle interno <-- CEID-CESI+1
	COPIA_EN	FESI_BIS,FESI  
	PON_A_CERO	VARIABLE_FESI	  
	PON_A_CERO      VARIABLE_CONT_BUC_INTERNO
	TRANSFIERE_A    CONT_BUC_INTERNO,CEID
	RESTA_A	CONT_BUC_INTERNO,CESI
	INCREMENTA	CONTADOR_BUC_INTERNO
	MOV     FACTOR_ITERACION_BUCLE_INT,CONTADOR_BUC_INTERNO
; Apuntar al inicio de la ventana, con DI, <-- (160xFESI)+(CESIx2)
	TRANSFIERE_A	AL,COLUMNAS*2
        MULTIPLICA      FESI_BIS 
;MULTIPLIC_A    CESI_BIS,POR_DOS
	SHL     CESI_BIS,POR_DOS
	SUMA_A	AX,CESI_BIS
	TRANSFIERE_A	PUNTERO_DE_VIDEO,AX
; Modo incremental del ndice DI:
	MODO_INCREMENTAL_DE_LOS_INDICES
; BP <-- factor de actualizacin de DI, para apuntar a siguiente
; lnea: BP <-- 160-((CEID-CESI+1)x2):  
	TRANSFIERE_A	FACTOR_INCREMENTO_DI,COLUMNAS*2	
;MULTIPLICA_A    CONTADOR_BUC_INTERNO,POR_DOS
	SHL     CONTADOR_BUC_INTERNO,POR_DOS
	RESTA_A FACTOR_INCREMENTO_DI,CONTADOR_BUC_INTERNO
; AX <-- carcter/atributo:
	SACA_DE_PILA	CARACTER_ATRIBUTO 
	TRANSFIERE_A	CARACTER,ESPACIO
	BORRAR_OTRA_FILA:
; recupera factor de iteracin del bucle interno: 
	MOV     CONTADOR_BUC_INTERNO,FACTOR_ITERACION_BUCLE_INT
BORRA_FILA:	
        REP STOSW               ;REP BORRA_ELEMENTO
; Apuntar al  inicio de la fila, con DI <-- DI+(160-(CEID-CESI+1)x2)
	SUMA_A  PUNTERO_DE_VIDEO,FACTOR_INCREMENTO_DI
; quedan filas por borrar?
	DECREMENTA CONTADOR_BUC_EXTERNO
	SI_NO_ES_CERO_VE_A BORRAR_OTRA_FILA
; NO: retornar:
	RETORNA

ENTRADA ENDP
CODIGO  ENDS
        END ENTRADA
