CODIGO  SEGMENT
        ASSUME  CS:CODIGO
ENTRADA PROC    FAR

COMMENT *
===================================================================
=                                                                 =
=                 PROCEDIMIENTO BORRA_VENTANA                     =
=                          Versin 5                              =
===================================================================


Descripcin: Borra una ventana definida dentro de la pantalla.

Condicin de llamada:
BH: Coordenada de la fila de la esquina superior izquierda
BL: Coordenada de la columna de la esquina superior izquierda
AH: Coordenada de la fila de la esquina inferior izquierda
AL: Coordenada de la columna de la esquina inferior izquierda
DH: Atributo que se asignar a la ventana borrada

Condicin de salida: 
C=0 si se ha borrado la ventana
C=1 si no se ha borrado la ventana por incoherencia de las  coordenadas
    pasadas como parmetros

Recursos usados y afectados:
AX: Para contener el par carcter/atributo
BX: Para ampliar a palabra el octeto CESI, en BL y, luego, como puntero
    base a la memoria de vdeo
CX: Para gestionar la iteracin del bucle interno
DL: Para gestionar la iteracin del bucle externo
DH: Para salvar temporalmente el parmetro BH
SI: Para contener el factor de iteracin del bucle interno
DI: Para indexar a lo largo de la lnea a borrar
BP: Para indicar el factor de actualizacin de DI, para que apunte al inicio
    de la siguiente lnea de la ventana
Registro de estado *

; -------------------------------------------------------------------

;Definicin de smbolos:
ESPACIO		EQU	20h
ATRIBUTO	EQU	07h
SEGMENTO_DE_VIDEO	EQU	0B800h
FILAS		EQU	25
COLUMNAS	EQU	80


BORRA_VENTANA:
; Comprobar la coherencia de los parmetros pasados:
        CMP     AH,BH                   ; FEID>=FESI?
	JB	INCOHERENTES
        CMP     AL,BL                   ; CEID>=CESI?
	JB	INCOHERENTES
        CMP     AH,FILAS                ; FEID>=25?
	JAE	INCOHERENTES
        CMP     AL,COLUMNAS             ; CEID>=80?
	JAE	INCOHERENTES
        CMP     BH,FILAS                ; FESI>=25?
	JAE	INCOHERENTES
        CMP     BL,COLUMNAS             ; CESI>=80?
	JNAE	COHERENTES
INCOHERENTES:
; Retornar indicando error de parmetros:
	STC
	RET
COHERENTES:
; Guarda el atributo pasado por DH:
        PUSH    DX              ; no se puede meter en la pila slo DH
; Inicia segmento de vdeo, en espacio de datos:
	MOV	DI,SEGMENTO_DE_VIDEO
	MOV     DS,DI
; Factor de iteracin del bucle externo <-- FEID-FESI+1:
	MOV	DL,AH
	SUB	DL,BH
	INC	DL
; Factor de iteracin del bucle interno <-- CEID-CESI+1
	MOV	DH,BH		; salva FESI, que se necesita luego
	SUB	BH,BH		; BH puede re-usarse (BL a palabra BX)
	SUB	CX,CX
	MOV	CL,AL
	SUB	CL,BL
	INC	CL
        MOV     SI,CX           ;  guarda factor de iteracin interno
; Apuntar al inicio de la ventana, con BX, <-- (160xFESI)+(CESIx2)
	MOV	AL,COLUMNAS*2
        MUL     DH              ; en DH se salv FESI, que estaba en BH
	SHL	BX,1
	ADD	AX,BX
	MOV	BX,AX
; AX <-- carcter/atributo:
	POP	AX			; recuperar en AH el atributo
	MOV	AL,ESPACIO
BORRA_OTRA_FILA:
; recupera factor de iteracin del bucle interno: 
	MOV	CX,SI
; Indexar al inicio de la lnea:	
	SUB	DI,DI	
BORRA_FILA:	
	MOV	[BX+DI],AX
	ADD	DI,2
	LOOP	BORRA_FILA
; Apuntar al  inicio de la siguiente fila, con BX <-- BX+160
	ADD	BX,COLUMNAS*2
; quedan filas por borrar?
	DEC	DL
	JNZ	BORRA_OTRA_FILA
; NO: retornar con C=0:
	RET

ENTRADA ENDP
CODIGO  ENDS
        END ENTRADA
